@@ -50,8 +50,8 @@ modelBaseClass <- setRefClass('modelBaseClass',
                                   getModelDef = function() modelDef,
                                   setModelDef = function(value) modelDef <<- value,
                                   getMaps = function(mapName, all = FALSE){
-                                  	if(all == TRUE)		return(modelDef$maps)
-                                  	return(modelDef$maps[[mapName]])
+                                      if(all == TRUE)		return(modelDef$maps)
+                                      return(modelDef$maps[[mapName]])
                                   },
                                   getCode = function() {
                                       '
@@ -59,9 +59,10 @@ Return the code for a model after\n,
 processing if-then-else statements,\n
 expanding macros, and replacing some\n
 keywords (e.g. nimStep for step) to \n
-avoid R ambiguity.'
+avoid R ambiguity.
+'
                                       modelDef$BUGScode
-                                   },
+                                  },
                                   isEndNode = function(nodes){  #Note: it says nodes, but graphIDs are fine too. Actually they are better
                                                                           '
 Determines whether one or more nodes are end nodes (nodes with no stochastic dependences)
@@ -93,7 +94,10 @@ Details: The return value is logical vector with an element for each node indica
 
                                   ## returns the declaration ID corresponding to nodes
                                   getDeclID = function(nodes) {
-                                      graphIDs <- modelDef$nodeName2GraphIDs(nodes, unique = FALSE)
+                                      graphIDs <- if(is.character(nodes))
+                                                      modelDef$nodeName2GraphIDs(nodes, unique = FALSE)
+                                                  else
+                                                      nodes
                                       declIDs <- getMaps('graphID_2_declID')[graphIDs]
                                       return(declIDs)
                                   },
@@ -130,10 +134,10 @@ nodes: A character vector specifying one or more node or variable names.
 
 Details: The return value is a character vector with an element for each node indicated in the input. Note that variable names are expanded to their constituent node names, so the length of the output may be longer than that of the input.
 '
-                                      nodeNames <- expandNodeNames(nodes, unique = FALSE)
+                                      nodeNames <- expandNodeNames(nodes, unique = FALSE, returnType = "ids")
                                       out <- sapply(nodeNames, function(x)
 				      	getDeclInfo(x)[[1]]$getDistributionName())
-                                      names(out) <- nodeNames
+                                      names(out) <- modelDef$maps$graphID_2_nodeName[nodeNames]
                                       return(out)
                                   },
 
@@ -535,57 +539,53 @@ Details: If a provided value (or the current value in the model when only a name
                                           ## but this simplifies the addition without changing exisiting code.
                                            data = list(...)
                                            ## Check if a single list or character vector was provided
-                                           if(length(data)==0) return()
-                                           if(length(data)==1)
-                                               if(is.null(data[[1]])) return()
-                                               if(is.character(data[[1]])) {
-                                                   if(length(data[[1]]) > 1)
+                                           if(length(data) == 0 || (length(data) == 1 && is.null(data[[1]]))) ## NULL catches case that occurs with test_size
+                                               return()
+                                           if(is.null(names(data))) {  ## if no names, should be either characters or single list
+                                               if(length(data) == 1) {
+                                                   if(is.character(data[[1]])) { # e.g., c('a','b','c')
                                                        data <- as.list(data[[1]])
-                                               } else {
-                                                   if(is.list(data[[1]])) {
-                                                       data <- data[[1]]
+                                                   } else {
+                                                       if(is.list(data[[1]])) {
+                                                           data <- data[[1]]
+                                                       } else stop("setData: single input should be a list or character strings")
                                                    }
-                                               }
-                                           if(length(data)==0) return()
-                                           ## When a variable name was provided, make it the list name and put the model's value for that variable as the list element
+                                               } else if(!all(sapply(data, function(x) is.character(x) && length(x) == 1)))
+                                                   stop("setData: multiple inputs must be named or be individual variable names")
+                                           } ##  otherwise, treat each element of ... as variable
                                            dataNames <- names(data)
                                            if(is.null(dataNames)) dataNames <- rep("", length(data))
                                            for(i in seq_along(data)) {
-                                               if(dataNames[i]=="") {
+                                               if(dataNames[i] == "" && is.character(data[[i]])) {
                                                    dataNames[i] <- data[[i]]
-                                                   data[[i]] <- get(dataNames[i])
+                                                   if(exists(dataNames[i]))
+                                                       data[[i]] <- get(dataNames[i])
                                                }
                                            }
                                            names(data) <- dataNames
-                                           data
 
-
-                                      ##  if(is.character(data)) {
-                                      ##     dataNames <- data
-                                      ##     data <- lapply(data, function(x) get(x))
-                                      ##     names(data) <- dataNames
-                                      ## }
                                       origData <<- data
                                       ## argument is a named list of data values.
                                       ## all nodes specified (except with NA) are set to that value, and have isDataEnv$VAR set to TRUE
-                                      if(length(data) == 0) return()
-                                      if(is.null(names(data)))
-                                          stop('\'data\' argument must by a named list')
 
-                                      ## Get names here because
-                                      ## unnecessary variables may be
-                                      ## stripped out of data in the
-                                      ## for loop.
-                                      namesData <- names(data)
                                       for(iData in seq_along(data)) {
-                                          varName <- namesData[iData]
+                                          varName <- dataNames[iData]
                                           varValue <- data[[varName]]
+                                          if(is.data.frame(varValue)) {
+                                              if(!all(sapply(varValue, is.numeric)))
+                                                  stop("setData: '", varName, "' must be numeric")
+                                              varValue <- as.matrix(varValue)
+                                              dimnames(varValue) <- NULL
+                                          }
                                           if(!(varName %in% names(isDataVars))) {
                                               ## when data is from modelDef$constantsList,
                                               ## it is possible that the constants don't exist on LHS of BUGS decls
                                               ## and so are not variables in the model.  In that case we don't want to issue the warning.
                                               if(warnAboutMissingNames
                                                  & nimbleOptions("verbose")) {
+                                                  if(varName == '') {
+                                                      warning('unnamed element provided to setData')
+                                                  } else 
                                                       warning('data not used in model: ',
                                                               varName)
                                               }
@@ -961,7 +961,7 @@ Checks for size/dimension mismatches and for presence of NAs in model variables
                                         for(v in .self$getVarNames()){
                                           if(!nimble:::isValid(.self[[v]])){
                                             message(' This model is not fully initialized. This is not an error. To see which variables are not initialized, use model$initializeInfo(). For more information on model initialization, see help(modelInitialization).', appendLF = FALSE)
-                                            break()
+                                            break
                                           }
                                         }
                                       }
@@ -1034,7 +1034,7 @@ Checks for errors in model specification and for missing values that prevent use
 
                                   },
 
-newModel = function(data = NULL, inits = NULL, modelName = character(), replicate = FALSE, check = getNimbleOption('checkModel')) {
+newModel = function(data = NULL, inits = NULL, modelName = character(), replicate = FALSE, check = getNimbleOption('checkModel'), calculate = TRUE) {
                                       '
 Returns a new R model object, with the same model definiton (as defined from the original model code) as the existing model object.
 
@@ -1044,24 +1044,27 @@ data: A named list specifying data nodes and values, for use in the newly return
 
 inits: A named list specifying initial values, for use in the newly returned model.  If not provided, the inits argument from the creation of the original R model object will be used.
 
-replicate: Logical specifying whether to repliate all current values and data flags from the current model in the new model.  If TRUE, then the data and inits arguments are not used.  Default value is FALSE.
-
-check: A logical indicating whether to check the model object for missing or invalid values.  Default is given by the NIMBLE option \'checkModel\', see help on \'nimbleOptions\' for details.
-
 modelName: An optional character string, used to set the internal name of the model object.  If provided, this name will propagate throughout the generated C++ code, serving to improve readability.
 
+replicate: Logical specifying whether to replicate all current values and data flags from the current model in the new model.  If TRUE, then the data and inits arguments are not used.  Default value is FALSE.
+
+check: A logical indicating whether to check the model object for missing or invalid values.  Default is given by the NIMBLE option \'checkModel\', see help on \'nimbleOptions\' for details. 
+
+calculate: A logical indicating whether to run \'calculate\' on the model; this will calculate all deterministic nodes and logProbability values given the current state of all nodes. Default is TRUE. For large models, one might want to disable this, but note that deterministic nodes, including nodes introduced into the model by NIMBLE, may be NA. 
+
 Details: The newly created model object will be identical to the original model in terms of structure and functionality, but entirely distinct in terms of the internal values.
 '
                                       if(replicate) {
-                                          newlyCreatedModel <- modelDef$newModel(check = FALSE)
+                                          newlyCreatedModel <- modelDef$newModel(check = FALSE, calculate = FALSE)
                                           nimCopy(from = .self, to = newlyCreatedModel, logProb = TRUE)
                                           for(var in ls(isDataEnv)) newlyCreatedModel$isDataEnv[[var]] <- isDataEnv[[var]]
                                           if(check) newlyCreatedModel$check()
+                                          if(calculate) newlyCreatedModel$calculate()
                                           return(newlyCreatedModel)
                                       }
                                       if(is.null(data)) data <- if( inherits(origData, 'uninitializedField') ) list() else origData
                                       if(is.null(inits)) inits <- if( inherits(origInits, 'uninitializedField') ) list() else origInits
-                                      modelDef$newModel(data = data, inits = inits, modelName = modelName, check = check)
+                                      modelDef$newModel(data = data, inits = inits, modelName = modelName, check = check, calculate = calculate)
                                   }
                               )
                               )
