@@ -158,17 +158,17 @@ makeParamInfo <- function(model, nodes, param) {
 
     distNames <- model$getDistribution(nodes)
 
-    ## if(length(nodes) != 1) stop(paste0("Problem with nodes argument while setting up getParam.  Should be length 1 but was: ", paste0(nodes, collapse = ",")))
-    ## distInfo <- getDistributionList(model$getDistribution(nodes))[[1]]
-    ## ## If nodes is invalid, an error from the above line will be trapped in parseEvalNumericMany
     if(length(param) != 1) stop(paste0(paste0('Problem with param(s) ', paste0(param, collapse = ','), ' while setting up getParam for node ', nodes,
-                 '\nOnly one parameter is allowed.')))
-    ## paramID <- distInfo$paramIDs[param]
-    ## if(length(paramID)!=1 | any(is.na(paramID))) stop(paste0('Problem with param ', paste0(param, collapse = ','), ' while setting up getParam for node ', nodes,
-    ##                                    '\nThe parameter name is not valid.'))
-    paramIDvec <- sapply(distNames, getParamID, param)
-    typeVec <- sapply(distNames, getType, param)
-    nDimVec <- sapply(distNames, getDimension, param)
+                                              '\nOnly one parameter is allowed.')))
+
+    distInfos <- lapply(distNames, getDistributionInfo)
+    paramIDvec <- unlist(lapply(distInfos, function(x) x$paramIDs[param]))
+    typeVec <- unlist(lapply(distInfos, function(x) x$types[[param]]$type))
+    nDimVec <- unlist(lapply(distInfos, function(x) x$types[[param]]$nDim))
+    
+   ## paramIDvec <- sapply(distNames, getParamID, param)
+   ## typeVec <- sapply(distNames, getType, param)
+   ## nDimVec <- sapply(distNames, getDimension, param)
     if(length(unique(typeVec)) != 1 || length(unique(nDimVec)) != 1) stop('cannot have an indexed vector of nodes used in getParam if they have different types or dimensions for the same parameter.') 
     ans <- c(list(paramID = paramIDvec), type = typeVec[1], nDim = nDimVec[1])
     class(ans) <- 'getParam_info'
@@ -926,7 +926,7 @@ nimCat <- function(...) {
 #' 
 #' When used in a \code{nimbleFunction} (in \code{run} or other member function), \code{numeric}, \code{integer} and \code{logical} are immediately converted to \code{nimNumeric}, \code{nimInteger} and \code{nimLogical}, respectively.  
 #' 
-#' @author Daniel Turek, Christopher Paciorek, Perry de Valpine
+#' @author Daniel Turek, Chris Paciorek, Perry de Valpine
 #' @aliases numeric
 #' @seealso \code{\link{nimMatrix}}, \code{\link{nimArray}}
 #' @export
